@@ -14,7 +14,7 @@ use File::Glob qw(:bsd_glob);
 use POSIX qw(strftime);                                                         # http://www.cplusplus.com/reference/ctime/strftime/
 use Data::Dump qw(dump);
 
-our $VERSION = '2017.313';
+our $VERSION = '2017.331';
 
 &saveToS3('DataTableText') if 0;                                                # Save code
 
@@ -45,6 +45,46 @@ sub fileSize($)
   (stat($file))[7]
  }
 
+
+#-------------------------------------------------------------------------------
+# Get the modified time of a file in seconds since epoch
+#-------------------------------------------------------------------------------
+
+sub fileModTime($)
+ {my ($file) = @_;
+  (stat($file))[9] // 0
+ }
+
+#-------------------------------------------------------------------------------
+# Check whether a target is out of date relative to an array of files.
+#-------------------------------------------------------------------------------
+
+=pod
+
+Used when one file is dependent on many files to make sure that the target is
+younger than all its sources. Allows for an easy test of the form:
+
+if (fileOutOfDate($target, $source1,$source2,$source3))
+
+as in 'make' to decide whether the target needs to be updated from its sources.
+
+Returns the first out of date source file to make debugging easier, or undef if
+no files are out of date.
+
+=cut
+
+sub fileOutOfDate($@)
+ {my ($target, @sources) = @_;                                                  # Target, sources
+  return $target unless -e $target;                                             # Out of date if target does not exist
+  my $t = fileModTime($target);                                                 # Time of target
+  for(@sources)                                                                 # Each source
+   {return $_ unless -e $_;                                                     # Out of date if source does not exist
+    my $T = fileModTime($_);                                                    # Time of source
+    return $_ if $T > $t;                                                       # Out of date if source newer than target
+   }
+  undef                                                                         # Not out of date as : target and all sources exist and target later than all of the sources
+ }
+
 #-------------------------------------------------------------------------------
 # Create a file path from file name components
 #-------------------------------------------------------------------------------
@@ -62,6 +102,13 @@ sub filePathDir(@)
   $f.'/';
  }
 
+sub filePathExt(@)                                                              # File with extension
+ {my $x = pop @_;
+  my $n = pop @_;
+  my $f = "$n.$x";
+  filePath(@_, $f)
+ }
+
 sub quoteFile($)                                                                # Quote a file name
  {my ($f) = @_;
   "\"$f\""
@@ -74,18 +121,19 @@ sub currentDirectory()
  }
 
 #-------------------------------------------------------------------------------
-# Make a path
+# Make a path for a file name or a folder
 #-------------------------------------------------------------------------------
 
 sub makePath($)
  {my ($path) = @_;
-  my @p = split /[\\\/]+/, $path; pop @p;
-  return 1 unless @p;
+  my @p = split /[\\\/]+/, $path;
+  return 1 unless @p > 1;
+  pop @p unless $path =~ /[\\\/]\Z/;
   my $p = join '/', @p;
   return 2 if -d $p;
   eval {make_path($p)};
   -d $p or confess "Cannot makePath $p";
-  1
+  0
  }
 
 #-------------------------------------------------------------------------------
@@ -610,9 +658,12 @@ use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
 @ISA          = qw(Exporter);
 @EXPORT       = qw(formatTable);
-@EXPORT_OK    = qw(appendFile checkKeys containingPowerOfTwo currentDirectory
+@EXPORT_OK    = qw(appendFile checkKeys containingPowerOfTwo
+convertImageToJpx currentDirectory
 dateStamp dateTimeStamp
-extractDocumentation fileList filePath filePathDir fileSize formatTableBasic
+extractDocumentation
+fileList fileModTime fileOutOfDate
+filePath filePathDir filePathExt fileSize formatTableBasic
 genLValueArrayMethods genLValueHashMethods
 genLValueScalarMethods genLValueScalarMethodsWithDefaultValues
 imageSize indentString
